from django import template

from django_spire.core.constants import TIME_UNITS_TO_SECONDS

register = template.Library()

@register.filter
def humanize_duration_simple(amount: float, start_unit: str = 'second') -> str:
    if not amount or amount == 0:
        return 'N/A'

    if start_unit not in TIME_UNITS_TO_SECONDS.keys():
        return 'N/A'

    amount = int(amount)

    if abs(amount) > 1:
        start_unit += 's'

    return f'{amount} {start_unit}'


@register.filter
def humanize_duration(amount: float, start_unit: str = 'second') -> str:
    if not amount or amount == 0:
        return 'N/A'

    if start_unit not in TIME_UNITS_TO_SECONDS.keys():
        return 'N/A'

    amount = amount * TIME_UNITS_TO_SECONDS[start_unit]

    components = []

    for unit, divisor in TIME_UNITS_TO_SECONDS.items():
        converted_amount, amount = divmod(int(amount), divisor)

        if converted_amount > 0:
            pluralize = '' if converted_amount == 1 else 's'
            duration = f'{converted_amount} {unit}{pluralize}'
            components.append(duration)

    return ', '.join(components)


from __future__ import annotations

import json
from typing import TYPE_CHECKING

from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import AnonymousUser
from django.template.response import TemplateResponse
from django.urls import reverse

from django_spire.contrib.generic_views.portal_views import infinite_scrolling_view
from django_spire.contrib.session.controller import SessionController
from django_spire.notification.app.constants import NOTIFICATION_FILTERING_SESSION_KEY_NAME
from django_spire.notification.app.forms import NotificationListFilterForm
from django_spire.notification.app.models import AppNotification
from django_spire.notification.choices import NotificationPriorityChoices, NotificationStatusChoices

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest
    from django.db.models import QuerySet


def _infinite_scroll_view(
    request: WSGIRequest,
    notification_queryset: QuerySet[AppNotification]
) -> TemplateResponse:
    body_data = json.loads(request.body.decode('utf-8'))

    return infinite_scrolling_view(
        request,
        queryset=notification_queryset,
        queryset_name='notifications',
        context_data={
            'app_notification_list_url': body_data.get('app_notification_list_url'),
        },
        template='django_spire/notification/app/scroll/item/items.html',
    )


@login_required()
def notification_infinite_scroll_view(request: WSGIRequest) -> TemplateResponse:
    if isinstance(request.user, AnonymousUser):
        notifications = AppNotification.objects.none()
    else:
        notifications = (
            AppNotification.objects.active()
            .is_sent()
            .annotate_is_viewed_by_user(request.user)
            .select_related('notification')
            .distinct()
            .ordered_by_priority_and_sent_datetime()
            .process_session_filter(
                request=request,
                session_key=NOTIFICATION_FILTERING_SESSION_KEY_NAME,
                form_class=NotificationListFilterForm,
            )
        )

    filter_session = SessionController(request, NOTIFICATION_FILTERING_SESSION_KEY_NAME)

    view = _infinite_scroll_view(request=request, notification_queryset=notifications)
    view.context_data.update({
        'filter_session': filter_session,
        'statuses': NotificationStatusChoices,
        'priorities': NotificationPriorityChoices,
    })
    return view


@login_required()
def dropdown_infinite_scroll_view(request: WSGIRequest) -> TemplateResponse:
    if isinstance(request.user, AnonymousUser):
        notifications = AppNotification.objects.none()
    else:
        notifications = (
            AppNotification.objects.active()
            .is_sent()
            .annotate_is_viewed_by_user(request.user)
            .select_related('notification')
            .distinct()
            .ordered_by_priority_and_sent_datetime()
        )

    view = _infinite_scroll_view(request=request, notification_queryset=notifications)
    view.context_data.update({
        'is_dropdown_item': True,
    })
    return view


@login_required()
def notification_dropdown_template_view(request: WSGIRequest) -> TemplateResponse:
    body_data = json.loads(request.body.decode('utf-8'))

    return TemplateResponse(
        request,
        context={
            'app_notification_list_url': body_data.get('app_notification_list_url'),
            'notification_endpoint': reverse(
                'django_spire:notification:app:template:dropdown_scroll_items'
            )
        },
        template='django_spire/notification/app/dropdown/notification_dropdown_content.html'
    )



from dataclasses import dataclass
from datetime import date
from django_spire.contrib.changelog.choices import ChangeLogTypeChoices


@dataclass
class Change:
    app: str
    description: str


@dataclass
class ChangelogEntry:
    version: str
    changes: list[Change]
    date: date
    type: ChangeLogTypeChoices



from __future__ import annotations

from django.urls import path

from django_spire.notification.app.views import template_views


app_name = 'django_spire_notification'

urlpatterns = [
    path(
        'notficiation/scroll/items',
        template_views.notification_infinite_scroll_view,
        name='scroll_items'
    ),
    path(
        'notficiation/dropdown/scroll/items',
        template_views.dropdown_infinite_scroll_view,
        name='dropdown_scroll_items'
    ),
    path(
        'notficiation/dropdown/template/',
        template_views.notification_dropdown_template_view,
        name='notification_dropdown'
    )
]



from __future__ import annotations

from typing import TYPE_CHECKING

from django.contrib import messages

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest
    from django.forms import BaseForm


def form_errors_as_list(*forms: BaseForm) -> list[str]:
    form_errors = []

    for form in forms:
        for field_name, error_list in form.errors.items():
            for error in error_list.data:
                error_message = ''

                if field_name != '__all__':
                    error_message += f'{field_name.title()}: '

                if hasattr(error, 'message_responses'):
                    error_message += f'{" ".join(error.message_responses)}'

                elif hasattr(error, 'messages'):
                    error_message += f'{" ".join(error.messages)}'

                else:
                    message = 'Error message not found.'
                    raise Exception(message)

                form_errors.append(error_message)

    return form_errors


def form_error_as_str(*form: BaseForm):
    error_message = ''

    for form in form:
        error_message += " ".join(form_errors_as_list(form))

    return error_message


def show_form_errors(request: WSGIRequest, *forms: BaseForm) -> None:
    for form in forms:
        for error in form_errors_as_list(form):
            messages.error(request=request, message=error)
